#!/bin/bash

# Execute updates.sh and updates.sql inside a migrations directory
# Error handling: if a migration produces an error, it will be skipped. A warning is given in the deployment log but the deployment continues
# If the directory contains a file named 'critical', then the deployment process will quit on error
# Usage:
#       applyThisMigration <migration directory>
#   e.g.:
#       applyThisMigration 2020-0625-1130

function applyThisMigration {
    cd "${projectPath}/Migrations"
    timestamp="$1"
    status=0
    registerText="${timestamp}"
    if [ -d "${timestamp}" ]; then

        if [ -f "${timestamp}/updates.sh" ]; then
            # Run the updates.sh script and pass $projectPath as parameter
            echo "$ run script 'Migrations/${timestamp}/updates.sh' from projectPath"
            cd "${projectPath}"
            /bin/bash "Migrations/${timestamp}/updates.sh" "${webDir}"
            status=$?
            registerText="${registerText} *sh"
            cd "${projectPath}/Migrations"
        fi

        if [[ $status -eq 0 ]] && [ -f "${timestamp}/updates.sql" ]; then
            importSqlFile "${timestamp}/updates.sql"
            status=$?
            registerText="${registerText} *sql"
        fi

        # If an error occured, and this migration is marked 'critical', then exit
        if [[ $status -ne 0 ]]; then
            if [ -f "${timestamp}/breakonerror" ] || [ -f "${timestamp}/critical" ]; then
                echo ">>>> ERROR: Migration ${timestamp} has not completely finished, see error above <<<<"
                echo ">>>>        This migration has been marked as 'critical', therefor exiting..... <<<<"
                kill -s TERM $TOP_PID
            fi
        fi

        if [[ $status -eq 0 ]]; then
            if [[ "${registerText}" == "${timestamp}" ]]; then
                echo ">>>> WARNING: nothing done for migration ${timestamp}, no updates.sql or updates.sh found! <<<<"
            fi
            # Register timestamp in 'MigrationsDone.txt' so that this migration won't be run again
            echo "${registerText}" >> MigrationsDone.txt
        else
            echo ">>>> ERROR: Migration ${timestamp} has not completely finished and will be attempted again in next run! <<<<"
            echo ">>>>        Deployment process continues, but please fix this migration and run a new deployment <<<<"
        fi

    fi
    return $status
}

# Apply all
function applyMigrations {
    cd "${projectPath}/Migrations"

    if [ -f "StartFrom" ]; then start=`cat StartFrom`; else start="0"; fi

    for D in *; do

        if [ -d "${D}" ] && [[ ! "${D}" < "${start}" ]]; then

            # Check if this migration has already been done
            grep "${D}" MigrationsDone.txt > /dev/null

            if [[ $? -ne 0 ]]; then
                action=1
                # Migration timestamp is not found in file 'Done', so execute this migration
	        	applyThisMigration "${D}"
		        echo
            fi
            # Just to make sure the path is not changed
    	    cd "${projectPath}/Migrations"
        fi
    done
    [[ $action -eq 1 ]] || echo "Nothing to do"
    cd ..
}

