# TYPO3 Installer scripts

_Loek Hilgersom, 25 june 2020_

## Default functions
- Update project code from git repository
- Run composer install (installs packages with versions as set in composer.lock)
- Perform install tool DB updates (using TYPO3 Console)
- Execute migrations, either Shell scripts or SQL commands * (see below)
- Clear TYPO3 caches (using TYPO3 Console)
- Compile SCSS files
- Run composer dumpautoload

## Usage
To run the deployment process on a remote server, log on to the system, check out git repository and run `./deploy.sh`
To get your development installation up-to-date after code updates, run `./local-deploy.sh`

##### deploy.sh
The only thing this script really does is `git pull` and then continue with `./local-deploy.sh`

The rest is error handling etc. (If git is not clean, a `git diff` is done to show the local changes
in the script output before they are overwritten by `git checkout -f`)

##### local-deploy.sh
This is the script where the real deployment steps are done. Modify and extend it to suit your needs.

### Using migrations

Migrations are database modifications or other changes that belong to a particular code-change/commit,
but cannot be done in the code itself, usually because it concerns content or files that are not under version control.

Migrations are stored in `Migrations/`. Every migration exists of a directory identified by a timestamp in the format
`2020-0625-1130`. Such a directory can contain the following files:
```
Migrations/
    2020-0625-1130/
        critical      # if this exists, the deployment process will stop on error
        updates.sh
        updates.sql
        <optional files used by updates.sh>
```
All files in a migration directory are optional. If a specific migration directory is empty, the deployment process will
register the migration as 'done', but also generate a warning.

Migrations are only executed once on every server environment the project runs. After succesful execution of a migration, 
the timestamp is registered in `Migrations/MigrationsDone.txt`. If the migration produces an error, a warning will be 
printed in the deployment output, but the process continues. If an error occurs in either the .sh or the .sql, then the 
migration timestamp will NOT be added to `Migrations/MigrationsDone.txt`. So fix the error, commit the changes, and 
run the deployment again to apply the migration again.

##### Filename 'critical'
Optional **critical**: If a migration is essential, e.g. because following migrations or changes depend on it,
then add an empty file named `critical` in the migration directory. If that exists, the deployment will stop on an
error in that migration.

##### Migration status register, specific for each environment
`Migrations/MigrationsDone.txt` contains timestamps of all migrations that have been executed on this system. This file
is specific for the environment and is therefor excluded from git versioning.

##### Modding
If, for some reason, you need to apply a migration again that has already been done, then simply remove the line
containing the identifier from `Migrations/MigrationsDone.txt`

If, for a development setup or e.g. after a migration to a new server, you want to exclude all migrations until a 
certain date/identifier, just add a file `Migrations/StartFrom` containing the minimum timestamp. 

##### Examples where migrations can be used:
- if an old extension has been replaced by a new one, and the data can be migrated automatically, put the SQL-queries
a file updates.sql in a new directory with current timestamp. Add this to commit containing the new extension, and
the data will be migrated automatically during deployment!
- after adding a new extension to your project in git, the static template needs to be added to the site's Typoscript template
in the database. This can be done with an SQL-update query.
- if the code refers to a specific content element which should be added automatically to some pages, migrations allow
you to insert the content element in the same commit as the code it belongs to.
- every now and then other actions are required, e.g. adding or removing some non-versioned files, or running a command.
For these occasions, updates.sh allows you e.g. to copy files from the migration directory to some place in the 
project files, run a command, etc.
