#!/bin/bash

# usage:
#   ./deploy.sh         for a full deployment run
#   ./local-deploy.sh   for local run, without git pull

# Initialize LibraryFunctions
source Installer/BaseFunctions.sh

#
# Start deployment actions from here
# You have these paths to work with: $projectPath, $webPath, and $EXT (for typo3conf/ext)
#

printheader "Check if there are any new commits to deploy, otherwise exit"
runnoexit git remote update
runnoexit git status -uno
git status -uno | grep "Your branch is behind" > /dev/null 2>&1
if [[ $? -eq 1 ]]; then
    echo -e "No new commits in remote branch, quit deployment script\n"
    exit 0
fi

printheader "Check if git is clean."
runnoexit git diff-index --quiet HEAD --
if [[ $? -ne 0 ]]; then
  printheader "git is not clean, run 'git diff' before forcing checkout"
  runnoexit git --no-pager diff
  printheader "Force checkout: 'git checkout -f'"
  run git checkout -f
fi

run git pull

source local-deploy.sh
