#!/bin/bash

# usage:
#   ./deploy.sh         for a full deployment run
#   ./local-deploy.sh   for local run, without git pull

echo

# Your settings
webDir="html"
# File for environment-specific TypoScript constants
localConstantsFile="${webDir}/local/config/constants-local.typoscript"
# Run "sass --help" for available options, e.g. "--update --style=compressed"
sassOptions="--style=compressed"

# Initialize LibraryFunctions
source Installer/Initialize.sh

#
# Start deployment actions from here
# You have these paths to work with: $projectPath, $webDir, and $EXT (for typo3conf/ext)
#

# TODO: With Migrations built-in, this can probably be removed
runOnce InstallerUploads/PreUpdates.sh

printheader "Install or update composer packages based on status in 'composer.lock'"
composer install --no-dev

printheader "TYPO3-console: update DB definitions"
run php8.2 ./vendor/bin/typo3 database:updateschema "*.add,*.change"

printheader "Execute migrations that are not yet executed on this environment"
run applyMigrations

printheader "TYPO3-console: clear TYPO3 caches"
run php8.2 ./vendor/bin/typo3 cache:flush

printheader "Compile SCSS files"

printheader "Reload composer autoloading cache"
composer dumpautoload

run replaceLocalConstantsSetting "deployment.time" "`date "+%Y-%m-%d %H:%M"`"
run replaceLocalConstantsSetting "deployment.lastCommit" "`git rev-parse --short HEAD`"

# TODO: With Migrations built-in, this can probably be removed
runOnce InstallerUploads/PostUpdates.sh

printheader "ALL DONE!"
